CraftBarSettings = {};

-- Bindings
BINDING_HEADER_CRAFTBARHEADER = "CraftBar";
BINDING_NAME_CRAFTBARSNAP = "MouseSnap";

local tradeskill_professions = {3908, 7411, 2656, 3100, 2259, 2108, 4036, 25229, 45357, 53428, 2550, 3273};
local tradeskill_spells =  {};

local tradeskill_buttons = {};

local MASQUE = LibStub("Masque", true);
local buttongroup = nil;

local initialized = false;

local buttonIndex = 1;
local craftbar_professions = {
    ['101Pick Lock'] = {spellid=1804},
    ['102Runeforging'] = {spellid=53428},
}

local craftbar_buttons = {};

function foo(str)
    local id = tonumber(str);
    local name =  GetSpellInfo(id);
    DEFAULT_CHAT_FRAME:AddMessage("spell id "..str..": "..(name or "nil"), 1.0, 1.0, 1.0, 1, 10);
end

function craftbar_spellid(spellName)
    local spellLink = GetSpellLink(spellName);
    if spellLink then
        return tonumber(spellLink:match("spell:(%d+)"));
    end
    
    return 0;
end

local debug = false;
function craftbar_debug(value, name)
    if (not debug) then
        return;
    end

    if (name) then
        DEFAULT_CHAT_FRAME:AddMessage("debug "..name..": "..(value or "nil"), 1.0, 1.0, 1.0, 1, 10);
    else
        DEFAULT_CHAT_FRAME:AddMessage("debug: "..(value or "nil"), 1.0, 1.0, 1.0, 1, 10);
    end
end

-- functions
function CraftBar_OnLoad(self)
    self:RegisterEvent("ADDON_LOADED");

    self:RegisterEvent("PLAYER_ENTER_COMBAT");
    self:RegisterEvent("PLAYER_LEAVE_COMBAT");
    self:RegisterEvent("PLAYER_REGEN_DISABLED");
    self:RegisterEvent("PLAYER_REGEN_ENABLED");

    self:RegisterEvent("TRADE_SKILL_SHOW");
    self:RegisterEvent("TRADE_SKILL_UPDATE");
    self:RegisterEvent("TRADE_SKILL_DATA_SOURCE_CHANGING");
    self:RegisterEvent("TRADE_SKILL_DATA_SOURCE_CHANGED");
    self:RegisterEvent("TRADE_SKILL_LIST_UPDATE");
    self:RegisterEvent("TRADE_SKILL_DETAILS_UPDATE");
    self:RegisterEvent("TRADE_SKILL_NAME_UPDATE");
    self:RegisterEvent("TRADE_SKILL_FILTER_UPDATE");

    CraftBarSettings.ShowTabs = CraftBarSettings.ShowTabs or true;
    CraftBarSettings.ShowRank = CraftBarSettings.ShowRank or false;

    --self:RegisterAllEvents();

    SLASH_CRAFTBAR1 = "/craftbar";
    SLASH_CRAFTBAR2 = "/craft";
    SlashCmdList["CRAFTBAR"] = function(msg)
        CraftBar_SlashCommandHandler(msg);
    end
end

function CraftBar_OnEvent(self, event, ...)

    --craftbar_debug(event, "event");

    if (event == "ADDON_LOADED") then

        local addonName = ...;
        
        if (addonName == "CraftBar") then
            if (MASQUE) then
                buttongroup = MASQUE:Group("CraftBar");
            end
        end
        
        if (addonName == "Blizzard_TradeSkillUI") then
            CraftBar_InitTradeSkillButtons();
            CraftBar_InitTradeSkillCasts();
        end
    end
    
    if ( event == "TRADE_SKILL_SHOW" or
         event == "TRADE_SKILL_UPDATE"  or
         event == "TRADE_SKILL_DATA_SOURCE_CHANGING"  or
         event == "TRADE_SKILL_DATA_SOURCE_CHANGED"  or
         event == "TRADE_SKILL_LIST_UPDATE"  or
         event == "TRADE_SKILL_DETAILS_UPDATE"  or
         event == "TRADE_SKILL_NAME_UPDATE"  or
         event == "TRADE_SKILL_FILTER_UPDATE" ) then
        CraftBar_ShowTradeSkillFrame();
    end

    if (event == "PLAYER_ENTER_COMBAT" or event == "TRADE_SKILL_UPDATE") then

        if (InCombatLockdown() == nil) then
            local frame = getglobal("CraftBarFrame");
            frame:Hide();
        end
    end
end

function CraftBar_SlashCommandHandler(msg)
    if (msg == "reload") then
        ReloadUI();
    elseif (msg == "toggletabs") then
        if (CraftBarSettings.ShowTabs) then
            CraftBarSettings.ShowTabs = false
        else
            CraftBarSettings.ShowTabs = true
        end
    elseif (msg == "togglerank") then
        if (CraftBarSettings.ShowRank) then
            CraftBarSettings.ShowRank = false
        else
            CraftBarSettings.ShowRank = true
        end
    else
        DEFAULT_CHAT_FRAME:AddMessage("/craftbar reload: reload UI", 1.0, 1.0, 1.0, 1, 10);
        DEFAULT_CHAT_FRAME:AddMessage("/craftbar toggletabs: show/hide side tabs at the TradeSkillFrame", 1.0, 1.0, 1.0, 1, 10);
        DEFAULT_CHAT_FRAME:AddMessage("/craftbar togglerank: show/hide rank on buttons", 1.0, 1.0, 1.0, 1, 10);
    end
end

function CraftBar_AppendProfessionInfo(index)
    if (index) then
        local name, texture, rank, maxRank, numSpells, spelloffset, skillLine, rankModifier, specializationIndex, specializationOffset = GetProfessionInfo(index);
    
        local spellid = craftbar_spellid(name);
        
        local index1 = spelloffset + 1;
        local index2 = spelloffset + 2;
        
        local _, id = GetSpellBookItemInfo(index1, BOOKTYPE_PROFESSION);
        
        if (id) then
            craftbar_professions[CraftBar_CreateKeyForName(name)] = {spellid=id, rank=rank, maxRank=maxRank};
        end        
                
        if (numSpells == 2) then
            local _, id2 = GetSpellBookItemInfo(index2, BOOKTYPE_PROFESSION);
                    
            if (id2 and not IsPassiveSpell(index2, BOOKTYPE_PROFESSION)) then
                name = GetSpellBookItemName(index2, BOOKTYPE_PROFESSION);
                
                tinsert(tradeskill_spells, id2);
            
                craftbar_professions[CraftBar_CreateKeyForName(name)] = {spellid=id2};
            end
        end
    end
end

function CraftBar_CreateKeyForName(name)
    local key = ""
    
    if (buttonIndex < 10) then
        key = "0"
    end
    
    local key = key..buttonIndex.."_"..name;
    
    buttonIndex = buttonIndex + 1;
    
    return key;
end

function CraftBar_initButtons()
    CraftBarFrame:Hide();
    
    -- init professions
    local prof1, prof2, arch, fish, cook, firstAid = GetProfessions();
    
    CraftBar_AppendProfessionInfo(prof1);
    CraftBar_AppendProfessionInfo(prof2);
    CraftBar_AppendProfessionInfo(arch);
    CraftBar_AppendProfessionInfo(fish);
    CraftBar_AppendProfessionInfo(cook);
    CraftBar_AppendProfessionInfo(firstAid);

    for name, data in craftbar_pairsByKeys(craftbar_professions) do
        local spellid = data.spellid;
        local rank = data.rank;
        
        local spellname, _, texture = GetSpellInfo(spellid);

        -- Button in der Buttonleiste
        local button = CreateFrame("Button", "CraftBar"..name, CraftBarFrame, "CraftBarButtonTemplate");

        button:RegisterForClicks("AnyUp");
        button.rangeTimer = -1;

        button:RegisterEvent("PLAYER_TARGET_CHANGED");
        button:RegisterEvent("ACTIONBAR_UPDATE_USABLE");
        button:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN");

        button:SetAttribute("type", "spell");
        button:SetAttribute("spell", spellname);
        button:SetAttribute("id", spellid);
        
        if (rank and CraftBarSettings.ShowRank) then
            button.Count:SetText(""..rank);
        end

        if (MASQUE) then 
            buttongroup:AddButton(button);
        end

        _G[button:GetName().."Icon"]:SetTexture(texture);

        craftbar_buttons[name] = button;
    end

    initialized = true;
end

function craftbar_pairsByKeys(t, f)
    local a = {};

    for n in pairs(t) do
        table.insert(a, n)
    end

    table.sort(a, f);
    local i = 0;      -- iterator variable
    local iter = function ()   -- iterator function
        i = i + 1;
        if a[i] == nil then
            return nil;
        else
            return a[i], t[a[i]];
        end
    end

    return iter;
end

function CraftBar_initCraftFrame()
    if (not initialized) then
        CraftBar_initButtons();
    end

    local count = 0;

    for name, data in craftbar_pairsByKeys(craftbar_professions) do
        local spellid = data.spellid;
        local spellname, rank, texture, cost, isFunnel, powerType, castTime, minRange, maxRange = GetSpellInfo(spellid);
        local usable, nomana = IsUsableSpell(spellname);

        local button = craftbar_buttons[name];

        if (usable) then
            button:ClearAllPoints();
            button:SetPoint("LEFT", (6 + 38 * count), 0);

            _G[button:GetName().."Icon"]:SetTexture(GetSpellTexture(spellname));

            button:Show();
            count = count + 1;
        else
            button:Hide();
        end
    end

    CraftBarFrame:SetWidth(10 + 38 * count);

    return count;
end

function CraftBar_SnapDown()

    if (InCombatLockdown()) then
        UIErrorsFrame:AddMessage(ERR_NOT_IN_COMBAT, 1.0, 0.0, 0.0, 1, 10);
    end

    local frame = getglobal("CraftBarFrame");

    local count = CraftBar_initCraftFrame();

    if (count < 1) then
        frame:Hide();
        return;
    end

    def_posx = frame:GetLeft();
    def_posy = frame:GetBottom();

    local x, y = GetCursorPosition();
    local scale = UIParent:GetEffectiveScale();

    x = x / scale;
    y = y / scale;

    frame:ClearAllPoints();
    frame:SetPoint("CENTER", UIParent, "BOTTOMLEFT", x, y);

    frame:Show();
end

function CraftBar_SnapUp()
    local frame = getglobal("CraftBarFrame");
    frame:ClearAllPoints();
    frame:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", def_posx, def_posy);

    if (not InCombatLockdown()) then
        frame:Hide();
    end
end

function CraftBar_MouseOver(button)
    CraftBar_SetTooltip(button);
end

function CraftBar_MouseOut(button)
    GameTooltip:FadeOut();
    selectedButton = nil;
end

function CraftBar_SetTooltip(button)

    local spellid = button:GetAttribute("id");

    GameTooltip_SetDefaultAnchor(GameTooltip, button);

    GameTooltip:SetSpellByID(spellid);

    GameTooltip:AppendText("");
end

function CraftBar_InitTradeSkillCasts()

    if (not CraftBarSettings.ShowTabs) then
        return;
    end
    
    local yoffset = 15;

    -- Feuerstelle
    CraftBar_InitTradeSkillCastForSpell(818, yoffset); -- 48
    
    yoffset = yoffset + 15;
    
    for id, spellid in craftbar_pairsByKeys(tradeskill_spells) do
        if (spellid ~= 818) then
            yoffset = yoffset + 48;
            CraftBar_InitTradeSkillCastForSpell(spellid, yoffset);
        end
    end
end

function CraftBar_InitTradeSkillCastForSpell(id, yoffset)
    local button = CreateFrame("Button", "CraftBarTradeSkill"..id, TradeSkillFrame, "CraftBarTradeSkillButtonTemplate");

    button:RegisterForClicks("AnyUp");
    button.rangeTimer = -1;

    button:RegisterEvent("PLAYER_TARGET_CHANGED");
    button:RegisterEvent("ACTIONBAR_UPDATE_USABLE");
    button:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN");
    
    local spellname, rank, texture = GetSpellInfo(id);

    button:SetAttribute("type", "spell");
    button:SetAttribute("spell", spellname);
    button:SetAttribute("id", id);
    
    button:SetNormalTexture(texture);
    button:SetPushedTexture(texture);
    
    button:ClearAllPoints();
    button:SetPoint("BOTTOMLEFT", TradeSkillFrame, "BOTTOMRIGHT", 0, yoffset);

    _G[button:GetName().."Icon"]:SetTexture(GetSpellTexture(spellname));
end

function CraftBar_InitTradeSkillButtons()

    if (not CraftBarSettings.ShowTabs) then
        return;
    end

    local button;
    
    local icon;

    for id, spellid in craftbar_pairsByKeys(tradeskill_professions) do
        local spellname, rank, texture, cost, isFunnel, powerType, castTime, minRange, maxRange = GetSpellInfo(spellid);

        -- Button in der Buttonleiste
        local button = CreateFrame("CheckButton", "CraftBarTradeSkillButton"..id, TradeSkillFrame, "CraftBarTradeSkillButtonTemplate");

        button:RegisterForClicks("AnyUp");
        button.rangeTimer = -1;

        button:RegisterEvent("PLAYER_TARGET_CHANGED");
        button:RegisterEvent("ACTIONBAR_UPDATE_USABLE");
        button:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN");

        button:SetAttribute("type", "spell");
        button:SetAttribute("spell", spellname);
        button:SetAttribute("id", spellid);
        
        button:SetNormalTexture(texture);
        button:SetPushedTexture(texture);

        tradeskill_buttons[id] = {button=button};
    end

    local count = 0;

    for id, spellid in craftbar_pairsByKeys(tradeskill_professions) do
        local spellname, rank, texture, cost, isFunnel, powerType, castTime, minRange, maxRange = GetSpellInfo(spellid);
        local usable, nomana = IsUsableSpell(spellname);

        local button = tradeskill_buttons[id].button;

        if (usable) then
            button:ClearAllPoints();
            button:SetPoint("TOPLEFT", TradeSkillFrame, "TOPRIGHT", 0, -(36 + 48 * count));

            _G[button:GetName().."Icon"]:SetTexture(GetSpellTexture(spellname));

            button:Show();
            count = count + 1;
        else
            button:Hide();
        end
    end
end

function CraftBar_ShowTradeSkillFrame()
    local tradeSkillID, skillLineName, skillLineRank, skillLineMaxRank, skillLineModifier =  C_TradeSkillUI.GetTradeSkillLine();
    
    local spell, button;
    for name, data in pairs(tradeskill_buttons) do
        button = data.button;
        spell = button:GetAttribute("spell");
        
        if (string.find(spell, skillLineName)) then
            button:SetChecked(true);
        else
            button:SetChecked(false);
        end
    end
end

function CraftBar_TradeSkillButtonMouseOver(button)
    local spellid = button:GetAttribute("id");

    GameTooltip_SetDefaultAnchor(GameTooltip, button);
    GameTooltip:SetSpellByID(spellid);

    GameTooltip:AppendText("");
end

function CraftBar_TradeSkillButtonMouseOut(button)
    GameTooltip:FadeOut();
    selectedButton = nil;
end
